
Unit Unit1;

{$mode objfpc}{$H+}

Interface

Uses
Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs, StdCtrls,
opbitmap, opbitmapformats, gif, gifdecoder, lazbridge, ExtCtrls;


Type

  { TForm1 }

  TForm1 = Class(TForm)
    Button1: TButton;
    Panel1: TPanel;
    Panel2: TPanel;
    Procedure Button1Click(Sender: TObject);
    Procedure FormCloseQuery(Sender: TObject; Var CanClose: boolean);
    Private
    { private declarations }
    Procedure GifUpdate(Sender: TObject);
    Public
    { public declarations }
  End;

{ TGifThread }

  TGifThread = Class(Tthread)
    Private
    fCanvas: Graphics.TCanvas;
    fFileName: string;
    fImage: TCanvasOpBitmap;
    fWidth: integer;
    fHeight: integer;
    fLeft: integer;
    fTop: integer;
    fOnUpdate: TNotifyEvent;
    fDisposalMethod: integer;
    Procedure UpdateDims(Desc: GifImageDescriptor);
    Public
    Procedure Execute;
    override;
    Procedure Update;
    property Image: TCanvasOpBitmap read fImage;
    property OnUpdate: TNotifyEvent read fOnUpdate write fOnUpdate;
    property FileName: string read fFileName write fFileName;
    property TrCanvas: Graphics.TCanvas read fCanvas write fCanvas;
    property GifDisposalMethod: integer read fDisposalMethod;
    property Left: integer read fLeft;
    property Top: integer read fTop;
    property Width: integer read fWidth;
    property Height: integer read fHeight;
  End;

Var
  Form1: TForm1;
  Thrd1, Thrd2: TGifThread;

  Implementation

{ TGifThread }

Procedure TGifThread.UpdateDims(Desc: GifImageDescriptor);
Begin
  fLeft := Desc.left_position;
  fTop := Desc.top_position;
  fWidth := Desc.image_width;
  fHeight := Desc.image_height;
End;

Procedure TGifThread.Execute;

Var gif: TGifDecoder;
  repeatmode: boolean;
Begin
  Try
    Try
      repeatmode := false;
      fimage := TCanvasOpBitmap.Create;
      gif := TGifDecoder.Create;
      gif.Verbose := false;
      gif.MultiImageMode := true;
      Repeat
        gif.Reset; ;
        gif.ReadImageFile(FileName, image);
        UpdateDims(gif.GifImgDescriptor);
        fImage.Transparent := TransparentColorFlag(gif.GraphicCtrlBlock);
        fImage.TransparentColor := fImage.ColorTable^[gif.GraphicCtrlBlock.transparent_color];

        If Not Terminated Then Synchronize(@Update);
        If gif.moreImages And gif.MultiImageMode Then repeatmode := true;
        While gif.moreImages And gif.MultiImageMode And (Not Terminated) Do
          Begin
            sleep(gif.GraphicCtrlBlock.delay_time * 10);
            If Not Terminated Then gif.readNextImage(image);
            fDisposalMethod := DisposalMethod(gif.GraphicCtrlBlock);
            UpdateDims(gif.GifImgDescriptor);
            If Not Terminated Then Synchronize(@Update);
          End;
      Until (Not repeatmode) Or Terminated;
    Except
      Terminate;
    End;
  Finally
    image.Destroy;
    gif.Destroy;
  End;
End;

Procedure TGifThread.Update;
Begin
  If Assigned(fOnUpdate) Then OnUpdate(Self);
End;


{ TForm1 }

Procedure TForm1.FormCloseQuery(Sender: TObject; Var CanClose: boolean);
Begin
  If Assigned(Thrd1) Then Thrd1.Terminate;
  If Assigned(Thrd2) Then Thrd2.Terminate;
End;


Procedure TForm1.Button1Click(Sender: TObject);

Var Path: string;
Begin
  If Button1.Tag = 0 Then
    Begin
      Button1.Tag := 1;
      Button1.Caption := 'Stop Animations';
      path := ExtractFilePath(application.ExeName);

      Thrd1 := TGifThread.create(true);
      With Thrd1 Do
        Begin
          OnUpdate := @GifUpdate;
          FreeOnTerminate := True;
          FileName := Path + 'logo.gif';
          TrCanvas := Panel1.Canvas;
          TrCanvas.Brush.color := clBtnFace;
          Resume;
        End;


      Thrd2 := TGifThread.create(true);
      With Thrd2 Do
        Begin
          OnUpdate := @GifUpdate;
          FreeOnTerminate := True;
          FileName := Path + 'ATT00005.gif';
          TrCanvas := Panel2.Canvas;
          TrCanvas.Brush.color := clBtnFace;
          Resume;
        End;

    End
  Else
    Begin
      Button1.Tag := 0;
      Button1.Caption := 'Start Animations';
      If Assigned(Thrd1) Then Thrd1.Terminate;
      If Assigned(Thrd2) Then Thrd2.Terminate;
      Thrd1 := Nil;
      Thrd2 := Nil;
    End;
End;


Procedure TForm1.GifUpdate(Sender: TObject);

Var Bmp: TBitmap;
Begin
  With TGifThread(Sender) Do
    Begin
      If GifDisposalMethod <> 1 Then TrCanvas.FillRect(0, 0, TrCanvas.Width - 1, TrCanvas.Height - 1);
      Bmp := TBitmap.create;
      AssignOpBitmapToBitmap(image, Bmp);
      Bmp.Transparent := Image.Transparent;
      TrCanvas.Draw(Left, Top, bmp);
      Bmp.free;
    End;
End;


initialization
  {$I unit1.lrs}

End.

